package it.unibas.questionari2.controllo;

import it.unibas.questionari2.modello.Questionario;
import it.unibas.questionari2.modello.Quesito;
import it.unibas.questionari2.modello.Risposte;
import it.unibas.questionari2.modello.Valutatore;

public class ControlloSecondario {
    
    public void schermoCompilaQuestionario(Risposte risposte, Questionario questionario) {
        if (questionario == null || risposte == null) {
            throw new IllegalStateException("Questionario nullo");
        }
        risposte.setIdQuestionario(questionario.getId());
        for (int i = 0; i < questionario.getNumeroQuesiti(); i++) {
            Quesito quesito = questionario.getQuesito(i);
            stampaQuesito(quesito, i);
            stampaPrecedenteRisposta(risposte, i);
            String risposta = leggiRisposta(quesito.getNumeroRisposte());
            if (risposta.equals("stop")) {
                return;
            }
            aggiornaRisposta(risposta, i, risposte);
        }
    }

    private void stampaQuesito(Quesito quesito, int i) {
        System.out.println("------------------------------------");
        System.out.println("         Quesito n. " + i);
        System.out.println("------------------------------------");
        System.out.println(quesito);
    }
    
    private void stampaPrecedenteRisposta(Risposte risposte, int i) {
        if (risposte.getNumeroRisposte() > i) {
            String risposta = risposte.getRisposta(i);
            if (!risposta.equals("")) {
                System.out.println("Risposta precedentemente fornita: " + risposta);
            }
        }
    }

    private String leggiRisposta(int numeroRisposte) {
        System.out.print("Risposta ? (\"stop\" per uscire) ----> ");
        String scelta = it.unibas.utilita.Console.leggiStringa();
        while (!scelta.equals("") && 
               !scelta.equals("stop") && 
               !verificaLettera(scelta, numeroRisposte)) {
            System.out.print("   Errore. Ripeti ----> ");
            scelta = it.unibas.utilita.Console.leggiStringa();
        }
        return scelta;
        
    }
    
    private boolean verificaLettera(String scelta, int numeroRisposte) {
        if (scelta.length() != 1) {
            return false;
        }
        int valore = (int)(scelta.charAt(0) - 97);
        return ((valore >= 0) && (valore < numeroRisposte));
    }
        
    private void aggiornaRisposta(String risposta, int i, Risposte risposte) {
        if (risposte.getNumeroRisposte() > i) {
            risposte.setRisposta(risposta, i);
        } else {
            risposte.addRisposta(risposta);
        }
    }

    public void schermoValutaQuestionario(Questionario questionario, Risposte risposte) {
        if (risposte == null || questionario == null) {
            throw new IllegalStateException("E' necessario compilare il questionario prima di valutarlo");
        }
        Valutatore valutatore = new Valutatore(questionario, risposte);
        System.out.println("\n" + risposte + "\n");
        System.out.println("Risposte esatte:    " + valutatore.getNumeroRisposteEsatte());
        System.out.println("Risposte non date:  " + valutatore.getNumeroRisposteNonDate());
        System.out.println("Risposte errate:    " + valutatore.getNumeroRisposteErrate());
    }    

}
