package it.unibas.questionari2.modello;

public class Valutatore {
    
    private Questionario questionario;
    private Risposte risposte;
    
    public Valutatore(Questionario questionario, Risposte risposte) {
        this.questionario = questionario;
        this.risposte = risposte;
    }

    public int getNumeroRisposte() {
        return this.risposte.getNumeroRisposte();
    }

    public String getRisposta(int i) {
        if (i >= this.getNumeroRisposte()) {
            throw new IndexOutOfBoundsException("Posizione inesistente");
        }
        return this.risposte.getRisposta(i);
    }

    public String getSoluzione(int i) {
        if (i >= this.questionario.getNumeroQuesiti()) {
            throw new IndexOutOfBoundsException("Posizione inesistente");
        }
        Quesito quesitoIesimo = this.questionario.getQuesito(i);
        return quesitoIesimo.getSoluzione();
    }

    public String getDifficolta(int i) {
        if (i >= this.getNumeroRisposte()) {
            throw new IndexOutOfBoundsException("Posizione inesistente");
        }
        Quesito quesitoIesimo = this.questionario.getQuesito(i);
        return quesitoIesimo.getIdDifficolta();
    }

    public boolean isEsatta(int i) {
        if (i >= this.getNumeroRisposte()) {
            throw new IndexOutOfBoundsException("Posizione inesistente");
        }
        return (this.getRisposta(i).equals(this.getSoluzione(i)));
    }
    
    public boolean isNonData(int i) {
        if (i >= this.getNumeroRisposte()) {
            throw new IndexOutOfBoundsException("Posizione inesistente");
        }
        return (this.getRisposta(i).equals(""));
    }
        
    public int getNumeroRisposteEsatte() {
        int risposteEsatte = 0;
        for (int i = 0; i < this.getNumeroRisposte(); i++) {
            if (this.getRisposta(i).equals(this.getSoluzione(i))) {
                risposteEsatte++;
            }
        }
        return risposteEsatte;
    }

    public int getNumeroRisposteErrate() {
        int risposteErrate = 0;
        for (int i = 0; i < this.getNumeroRisposte(); i++) {
            if (!this.getRisposta(i).equals("") &&
                !this.getRisposta(i).equals(this.getSoluzione(i))) {
                risposteErrate++;
            }
        }
        return risposteErrate;
    }    
    
    public int getNumeroRisposteNonDate() {
        int quesiti = questionario.getNumeroQuesiti();
        int risposteDate = this.getNumeroRisposteEsatte() + this.getNumeroRisposteErrate();
        return quesiti - risposteDate;
    }    
    
}
