package it.unibas.questionari2.persistenza;

import it.unibas.questionari2.modello.Questionario;
import it.unibas.questionari2.modello.Argomento;
import it.unibas.questionari2.modello.Difficolta;
import it.unibas.questionari2.modello.Quesito;

public class DAOQuestionario {
    
    public static Questionario caricaQuestionario(String nomeFile) throws DAOException {
        org.w3c.dom.Document document = costruisciDOM(nomeFile);
        Questionario questionario = new Questionario();
        analizzaRadice(document, questionario);
        riempiArgomenti(document, questionario);
        riempiDifficolta(document, questionario);
        riempiQuesiti(document, questionario);
        return questionario;
    }
    
    private static org.w3c.dom.Document costruisciDOM(String nomeFile) throws DAOException {
        org.w3c.dom.Document document = null;
        javax.xml.parsers.DocumentBuilderFactory factory = javax.xml.parsers.DocumentBuilderFactory.newInstance();
        factory.setValidating(true);
        factory.setNamespaceAware(true);
        MioErrorHandler mioErrorHandler = new MioErrorHandler();
        try {
            javax.xml.parsers.DocumentBuilder builder = factory.newDocumentBuilder();
            builder.setErrorHandler(mioErrorHandler);
            java.io.File file = new java.io.File(nomeFile);
            document = builder.parse(file);
        } catch (javax.xml.parsers.ParserConfigurationException pce) {
            //pce.printStackTrace();
            throw new DAOException(pce);
        } catch (org.xml.sax.SAXException sxe) {
            //sxe.printStackTrace();
            throw new DAOException(sxe);
        } catch (java.io.IOException ioe) {
            //ioe.printStackTrace();
            throw new DAOException(ioe);
        }
        if (!mioErrorHandler.isValido()) {
            throw new DAOException(mioErrorHandler.getErrori());
        }
        return document;
    }
    
    private static void analizzaRadice(org.w3c.dom.Document document, Questionario questionario) {
        org.w3c.dom.Element radice = document.getDocumentElement();
        String id = radice.getAttribute("id");
        String nomeDisciplina = radice.getAttribute("disciplina");
        questionario.setId(id);
        questionario.setDisciplina(nomeDisciplina);
    }
    
    private static void riempiArgomenti(org.w3c.dom.Document document, Questionario questionario) {
        org.w3c.dom.Element radice = document.getDocumentElement();
        org.w3c.dom.Element listaArgomenti = getFirstChildByName(radice, "listaArgomenti");
        org.w3c.dom.NodeList argomenti = listaArgomenti.getElementsByTagName("argomento");
        for(int i = 0; i < argomenti.getLength(); i++){
            org.w3c.dom.Element elementoArgomento = (org.w3c.dom.Element)argomenti.item(i);
            Argomento argomento = riempiArgomento(elementoArgomento);
            questionario.addArgomento(argomento);
        }
    }
    
    private static Argomento riempiArgomento(org.w3c.dom.Element elementoArgomento) {
        String id = elementoArgomento.getAttribute("id");
        String nome = elementoArgomento.getAttribute("nome");
        Argomento argomento = new Argomento();
        argomento.setId(id);
        argomento.setNome(nome);
        return argomento;
    }
    
    private static void riempiDifficolta(org.w3c.dom.Document document, Questionario questionario) {
        org.w3c.dom.Element radice = document.getDocumentElement();
        org.w3c.dom.Element listaDifficolta = getFirstChildByName(radice, "listaDifficolta");
        org.w3c.dom.NodeList nldifficolta = listaDifficolta.getElementsByTagName("difficolta");
        for(int i = 0; i < nldifficolta.getLength(); i++){
            org.w3c.dom.Element elementoDifficolta = (org.w3c.dom.Element)nldifficolta.item(i);
            Difficolta difficolta = riempiDifficolta(elementoDifficolta);
            questionario.addDifficolta(difficolta);
        }
    }
    
    private static Difficolta riempiDifficolta(org.w3c.dom.Element elementoDifficolta) {
        String id = elementoDifficolta.getAttribute("id");
        String descrizione = elementoDifficolta.getAttribute("descrizione");
        Difficolta difficolta = new Difficolta();
        difficolta.setId(id);
        difficolta.setDescrizione(descrizione);
        return difficolta;
    }
    
    private static void riempiQuesiti(org.w3c.dom.Document document, Questionario questionario) {
        org.w3c.dom.Element radice = document.getDocumentElement();
        org.w3c.dom.Element elencoQuesiti = getFirstChildByName(radice, "listaQuesiti");
        org.w3c.dom.NodeList quesiti = elencoQuesiti.getElementsByTagName("quesito");
        for(int i = 0; i < quesiti.getLength(); i++){
            org.w3c.dom.Element elementoQuesito = (org.w3c.dom.Element)quesiti.item(i);
            Quesito quesito = riempiQuesito(elementoQuesito);
            questionario.addQuesito(quesito);
        }
    }
    
    private static Quesito riempiQuesito(org.w3c.dom.Element elementoQuesito) {
        String id = elementoQuesito.getAttribute("id");
        String idArgomento = elementoQuesito.getAttribute("argomento");
        String idDifficolta = elementoQuesito.getAttribute("difficolta");
        Quesito quesito = new Quesito();
        quesito.setId(id);
        quesito.setIdArgomento(idArgomento);
        quesito.setIdDifficolta(idDifficolta);
        riempiDomanda(elementoQuesito, quesito);
        riempiRisposte(elementoQuesito, quesito);
        return quesito;
    }
    
    private static void riempiDomanda(org.w3c.dom.Element elementoQuesito, Quesito quesito) {
        org.w3c.dom.Element elementoDomanda = getFirstChildByName(elementoQuesito, "domanda");
        org.w3c.dom.Text testoDomanda = (org.w3c.dom.Text)elementoDomanda.getFirstChild();
        String valore = testoDomanda.getNodeValue().trim();
        quesito.setDomanda(valore);
    }
    
    private static void riempiRisposte(org.w3c.dom.Element elementoQuesito, Quesito quesito) {
        org.w3c.dom.Element elencoRisposte = getFirstChildByName(elementoQuesito, "listaRisposte");
        String soluzione = elencoRisposte.getAttribute("soluzione");
        quesito.setSoluzione(soluzione);
        org.w3c.dom.NodeList risposte = elencoRisposte.getElementsByTagName("risposta");
        for(int i = 0; i < risposte.getLength(); i++){
            org.w3c.dom.Element elementoRisposta = (org.w3c.dom.Element)risposte.item(i);
            org.w3c.dom.Text testoRisposta = (org.w3c.dom.Text)elementoRisposta.getFirstChild();
            String risposta = testoRisposta.getNodeValue().trim();
            quesito.addRisposta(risposta);
        }
    }
    
    private static org.w3c.dom.Element getFirstChildByName(org.w3c.dom.Element elemento, String nome) {
        org.w3c.dom.NodeList figli = elemento.getChildNodes();
        for (int i = 0; i < figli.getLength(); i++) {
            org.w3c.dom.Node figlio = figli.item(i);
            if (figlio.getNodeName().equals(nome)) {
                return (org.w3c.dom.Element)figlio;
            }
        }
        return null;
    }
    
}
