package it.unibas.questionari2.test;

import it.unibas.questionari2.modello.Questionario;
import it.unibas.questionari2.modello.Quesito;
import it.unibas.questionari2.modello.Argomento;
import it.unibas.questionari2.modello.Difficolta;
import it.unibas.questionari2.persistenza.DAOQuestionario;
import it.unibas.questionari2.persistenza.DAOException;

public class TestDAOQuestionario extends junit.framework.TestCase {

    private String nomeFile1 = Utilita.nomeFile1;
    private String nomeFile2 = Utilita.nomeFile2;
    private String nomeFile3 = Utilita.nomeFile3;
    private String nomeFile4 = Utilita.nomeFile4;
    private Questionario questionario;
    
    protected void setUp() throws java.lang.Exception {
    }

    protected void tearDown() throws java.lang.Exception {
    }
    
    public void testCaricaQuestionario1() {
        Questionario questionario = null;
        try {
            questionario = DAOQuestionario.caricaQuestionario(nomeFile1);
        } catch (Exception e) {
            e.printStackTrace();
            fail();
        }
        verificaQuestionario(questionario, "test1", "Disciplina di prova", 1, 3, 1);
    }
            
    public void testCaricaQuestionario2() {
        Questionario questionario = null;
        try {
            questionario = DAOQuestionario.caricaQuestionario(nomeFile2);
        } catch (DAOException daoe) {
            System.err.println(daoe);
            fail();
        }
        verificaQuestionario(questionario, "test4", "Geografia", 3, 3, 4);
    }

    public void testCaricaQuestionario3() {
        Questionario questionario = null;
        try {
            questionario = DAOQuestionario.caricaQuestionario(nomeFile3);
            fail();
        } catch (DAOException daoe) {
            assertTrue(daoe != null);
            //System.err.println(daoe);
        }
    }

    public void testCaricaQuestionario5() {
        Questionario questionario = null;
        try {
            questionario = DAOQuestionario.caricaQuestionario(nomeFile4);
        } catch (DAOException daoe) {
            System.err.println(daoe);
            fail();
        }
        verificaQuestionario(questionario, "test9", "Geografia", 3, 3, 9);
    }

    
    public void testCaricaArgomento1() {
        Questionario questionario = null;
        try {
            questionario = DAOQuestionario.caricaQuestionario(nomeFile1);
        } catch (DAOException daoe) {
            System.err.println(daoe);
            fail();
        }
        Argomento argomento = questionario.getArgomento("yyy");
        verificaArgomento(questionario, argomento, "yyy", "YYY");
    }

    public void testCaricaArgomento2() {
        Questionario questionario = null;
        try {
            questionario = DAOQuestionario.caricaQuestionario(nomeFile4);
        } catch (DAOException daoe) {
            System.err.println(daoe);
            fail();
        }
        Argomento argomento = questionario.getArgomento("naz");
        verificaArgomento(questionario, argomento, "naz", "Nazioni Europee");
    }
    
    public void testCaricaDifficolta1() {
        Questionario questionario = null;
        try {
            questionario = DAOQuestionario.caricaQuestionario(nomeFile1);
        } catch (DAOException daoe) {
            System.err.println(daoe);
            fail();
        }
        Difficolta difficolta = questionario.getDifficolta("d1");
        verificaDifficolta(questionario, difficolta, "d1", "Domanda di difficolta' media");
        difficolta = questionario.getDifficolta("d2");
        verificaDifficolta(questionario, difficolta, "d2", "Domanda di difficolta' avanzata");
    }

    public void testCaricaDifficolta2() {
        Questionario questionario = null;
        try {
            questionario = DAOQuestionario.caricaQuestionario(nomeFile4);
        } catch (DAOException daoe) {
            System.err.println(daoe);
            fail();
        }
        Difficolta difficolta = questionario.getDifficolta("d1");
        verificaDifficolta(questionario, difficolta, "d1", "Domanda di difficolta' media");
        difficolta = questionario.getDifficolta("d2");
        verificaDifficolta(questionario, difficolta, "d2", "Domanda di difficolta' avanzata");
    }

    public void testCaricaQuesito1() {
        Questionario questionario = null;
        try {
            questionario = DAOQuestionario.caricaQuestionario(nomeFile1);
        } catch (DAOException daoe) {
            System.err.println(daoe);
            fail();
        }
        Quesito quesito = questionario.getQuesito(0);
        verificaQuesito(quesito, "xxx", "Questo e' il testo della domanda ?", 4,
                        "aaa", "bbb", "ccc", "b", "yyy", "d1");
        assertTrue("risposta d", quesito.getRisposta(3).equals("ddd"));
        try {
            quesito.getRisposta(4);
            fail();
        } catch (Exception e) {
            assertTrue("risposta e",  e != null);
        }
    }

    public void testCaricaQuesito2() {
        Questionario questionario = null;
        try {
            questionario = DAOQuestionario.caricaQuestionario(nomeFile4);
        } catch (DAOException daoe) {
            System.err.println(daoe);
            fail();
        }
        Quesito quesito = questionario.getQuesito(2);
        verificaQuesito(quesito, "cap03", "Qual e' la capitale del Regno Unito ?", 3,
                        "Il Cairo", "Roma", "Londra - esatta", "c", "cap", "d2");
    }    

    public void verificaQuestionario(Questionario questionario, 
                                        String id, String disciplina,
                                        int numeroArgomenti, int numeroDifficolta, 
                                        int numeroQuesiti) {
        assertTrue("id", questionario.getId().equals(id));
        assertTrue("disciplina", questionario.getDisciplina().equals(disciplina));
        assertTrue("numero argomenti", questionario.getNumeroArgomenti() == numeroArgomenti);
        assertTrue("numero difficolta", questionario.getNumeroDifficolta() == numeroDifficolta);
        assertTrue("numero quesiti", questionario.getNumeroQuesiti() == numeroQuesiti);
    }
        
    public void verificaArgomento(Questionario questionario,
                                    Argomento argomento, String id, 
                                    String nome) {
        assertTrue("id", argomento.getId().equals(id));
        assertTrue("nome argomento", argomento.getNome().equals(nome));                                         
    }

    public void verificaDifficolta(Questionario questionario,
                                    Difficolta difficolta, String id,
                                    String descrizione) {
        assertTrue("id", difficolta.getId().equals(id));
        assertTrue("descrizione", difficolta.getDescrizione().equals(descrizione)); 
    }
    
    public void verificaQuesito(Quesito quesito, String id, 
                                    String domanda, int numeroRisposte,
                                    String rispostaA, String rispostaB,
                                    String rispostaC, String soluzione,
                                    String idArgomento, String idDifficolta) {
        assertTrue("id quesito", quesito.getId().equals(id)); 
        assertTrue("domanda: ", quesito.getDomanda().equals(domanda));
        assertTrue("numero risposte", quesito.getNumeroRisposte() == numeroRisposte);
        assertTrue("risposta a", quesito.getRisposta(0).equals(rispostaA));
        assertTrue("risposta b", quesito.getRisposta(1).equals(rispostaB));
        assertTrue("risposta c", quesito.getRisposta(2).equals(rispostaC));
        assertTrue("risposta esatta", quesito.getSoluzione().equals(soluzione));
        String idArgomentoQ = quesito.getIdArgomento();
        assertTrue("argomento quesito", idArgomentoQ.equals(idArgomento));        
        String idDifficoltaQ = quesito.getIdDifficolta();
        assertTrue("difficolta", idDifficoltaQ.equals(idDifficolta));        
    }
    
}
